@@ -4,6 +4,7 @@ from django.conf import settings
 from django.core.files.base import ContentFile
 from django.db import models
 from django.db.models.base import ModelBase
+from django.utils.html import conditional_escape as escape
 from django.utils.translation import ugettext_lazy as _
 
 from imagekit import specs
@@ -31,10 +32,10 @@ CROP_VERT_CHOICES = (
 
 class ImageModelBase(ModelBase):
     """ ImageModel metaclass
-    
+
     This metaclass parses IKOptions and loads the specified specification
     module.
-    
+
     """
     def __init__(cls, name, bases, attrs):
         parents = [b for b in bases if isinstance(b, ImageModelBase)]
@@ -46,7 +47,7 @@ class ImageModelBase(ModelBase):
             module = __import__(opts.spec_module,  {}, {}, [''])
         except ImportError:
             raise ImportError('Unable to load imagekit config module: %s' % \
-                opts.spec_module)    
+                opts.spec_module)
         for spec in [spec for spec in module.__dict__.values() \
                      if isinstance(spec, type) \
                      and issubclass(spec, specs.ImageSpec) \
@@ -58,20 +59,20 @@ class ImageModelBase(ModelBase):
 
 class ImageModel(models.Model):
     """ Abstract base class implementing all core ImageKit functionality
-    
+
     Subclasses of ImageModel are augmented with accessors for each defined
     image specification and can override the inner IKOptions class to customize
     storage locations and other options.
-    
+
     """
     __metaclass__ = ImageModelBase
 
     class Meta:
         abstract = True
-        
+
     class IKOptions:
         pass
-        
+
     def admin_thumbnail_view(self):
         if not self._imgfield:
             return None
@@ -82,17 +83,21 @@ class ImageModel(models.Model):
         else:
             if hasattr(self, 'get_absolute_url'):
                 return u'<a href="%s"><img src="%s"></a>' % \
-                    (self.get_absolute_url(), prop.url)
+                    (escape(self.get_absolute_url()), escape(prop.url))
             else:
                 return u'<a href="%s"><img src="%s"></a>' % \
-                    (self._imgfield.url, prop.url)
+                    (escape(self._imgfield.url), escape(prop.url))
     admin_thumbnail_view.short_description = _('Thumbnail')
     admin_thumbnail_view.allow_tags = True
-    
+
     @property
     def _imgfield(self):
         return getattr(self, self._ik.image_field)
 
+    @property
+    def _storage(self):
+        return getattr(self._ik, 'storage', self._imgfield.storage)
+
     def _clear_cache(self):
         for spec in self._ik.specs:
             prop = getattr(self, spec.name())
@@ -104,17 +109,26 @@ class ImageModel(models.Model):
                 prop = getattr(self, spec.name())
                 prop._create()
 
+    def save_image(self, name, image, save=True, replace=True):
+        if self._imgfield and replace:
+            self._imgfield.delete(save=False)
+        if hasattr(image, 'read'):
+            data = image.read()
+        else:
+            data = image
+        content = ContentFile(data)
+        self._imgfield.save(name, content, save)
+
     def save(self, clear_cache=True, *args, **kwargs):
-        is_new_object = self._get_pk_val is None
+        is_new_object = self._get_pk_val() is None
         super(ImageModel, self).save(*args, **kwargs)
-        if is_new_object:
+        if is_new_object and self._imgfield:
             clear_cache = False
             spec = self._ik.preprocessor_spec
             if spec is not None:
                 newfile = self._imgfield.storage.open(str(self._imgfield))
                 img = Image.open(newfile)
-                img = spec.process(img, None)
-                format = img.format or 'JPEG'
+                img, format = spec.process(img, self)
                 if format != 'JPEG':
                     imgfile = img_to_fobj(img, format)
                 else:
@@ -126,8 +140,9 @@ class ImageModel(models.Model):
                 name = str(self._imgfield)
                 self._imgfield.storage.delete(name)
                 self._imgfield.storage.save(name, content)
-        if clear_cache and self._imgfield != '':
-            self._clear_cache()
+        if self._imgfield:
+            if clear_cache:
+                self._clear_cache()
             self._pre_cache()
 
     def delete(self):
